// TrafficLight.cpp
// Demo control class implementation file

#include "StdAfx.h"

#include "TrafficLight.h"

using namespace TrafficLightDemo;

using namespace System::Drawing;
using namespace System::IO;

TrafficLight::TrafficLight()
{
  // With relative file names like this, the program expects the image files in the current
  // working directory. This is the project directory (the one with the source files) when
  // it is started inside the VS IDE with default project settings.

  // The bitmap images used in this demo are based on
  // http://en.wikipedia.org/wiki/File:Traffic_light.gif

  BackgroundImage = Image::FromFile("TrafficLightDark.png");
  m_pbxRed = CreateLightPbx("LightRed.png", 7);
  m_pbxYellow = CreateLightPbx("LightYellow.png", 43);
  m_pbxGreen = CreateLightPbx("LightGreen.png", 79);
  m_atlsCycleArray = GetCycleArray();
}

PictureBox ^TrafficLight::CreateLightPbx(String ^strFileName, int nTop)
{
  PictureBox ^pbx = gcnew PictureBox;
  pbx->Image = Image::FromFile(strFileName);
  pbx->Name = Path::GetFileNameWithoutExtension(strFileName);
  pbx->SizeMode = PictureBoxSizeMode::AutoSize;
  pbx->Location = Point(0, nTop);
  pbx->Visible = false;
  Controls->Add(pbx);
  pbx->Click += gcnew EventHandler(this, &TrafficLight::LightPbxClickHandler);
  return pbx;
}

TrafficLightState TrafficLight::State::get()
{
  return m_state;
}

void TrafficLight::State::set(TrafficLightState value)
{
  m_state = value;
  m_pbxRed->Visible = value.HasFlag(TrafficLightState::Red);
  m_pbxYellow->Visible = value.HasFlag(TrafficLightState::Yellow);
  m_pbxGreen->Visible = value.HasFlag(TrafficLightState::Green);
}

void TrafficLight::LightPbxClickHandler(Object ^sender, EventArgs ^e)
{
  // Forward picture box clicks to the TrafficLight (i.e. myself)
  // because visible picture boxes are mouse-opaque:

  OnClick(e);
}

// This implementation returns a traffic light cycle as used in Germany.
// The function is declared protected virtual, so derived classes can
// override it to change the cycle.

array<TrafficLightState> ^TrafficLight::GetCycleArray()
{
  return gcnew array<TrafficLightState>{
      TrafficLightState::Green, TrafficLightState::Yellow,
      TrafficLightState::Red, TrafficLightState::RedYellow
    };
}

void TrafficLight::AdvanceCycle()
{
  m_nCycleIndex = (m_nCycleIndex + 1) % m_atlsCycleArray->Length;
  State = m_atlsCycleArray[m_nCycleIndex];
}
